-- PalletSES class
-- Adds the possibility to define a sowing machine trigger and a dynamic mount trigger similar to the pallet and bale objects
-- Requires the Mountable specialization to be present (for dynamic mounting functionality)
--
-- @author  Stefan Geiger
-- @date  25/02/13		
--
-- @Edit: Bluebaby210: to fill fertilizer and bulid it in trailer
-- @date  31.07.2013
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

FertilizerFill = {};

function FertilizerFill.prerequisitesPresent(specializations)
    return true;
end;

function FertilizerFill:load(xmlFile)

    local sprayerFillTriggerId = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fertilizerFillTrigger#index"));
    if sprayerFillTriggerId ~= nil then
        self.sprayerFillTrigger = FertilizerFillTrigger:new(sprayerFillTriggerId);
    end;
end;

function FertilizerFill:delete()
    if self.sprayerFillTrigger ~= nil then
        self.sprayerFillTrigger:delete();
    end;
end;

function FertilizerFill:mouseEvent(posX, posY, isDown, isUp, button)
end;

function FertilizerFill:keyEvent(unicode, sym, modifier, isDown)
end;

function FertilizerFill:update(dt)
end;

function FertilizerFill:updateTick(dt)
end;

function FertilizerFill:draw()
end;

function FertilizerFill:dynamicMountTriggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
end;

-- PalletSESFillTrigger class
-- Replacement for the original SowingMachineFillTrigger class which has a support lower price and supports dynamic deleting

FertilizerFillTrigger = {}

local FertilizerFillTrigger_mt = Class(FertilizerFillTrigger);

function FertilizerFillTrigger:new(nodeId, mt)
    if mt == nil then
        mt = FertilizerFillTrigger_mt;
    end
    local self = {};
    setmetatable(self, mt);

    self.triggerId = nodeId;
    addTrigger(nodeId, "triggerCallback", self);
	
	--self.neededFruitType = getUserAttribute(nodeId, "neededFruitType"); fertilizer
    self.triggeredSowingMachines = {};
	local sprayTypeStr = getUserAttribute(nodeId, "neededFruitType");
		if sprayTypeStr ~= nil then
			self.sprayTypeDesc = Sprayer.sprayTypes[sprayTypeStr];
			if self.sprayTypeDesc ~= nil then
			local fillType = Sprayer.sprayTypeToFillType[self.sprayTypeDesc.index];
				if fillType ~= nil then
					self.fillType = fillType;
				end;
			end;
		end;
    return self;
end;

function FertilizerFillTrigger:delete()
    removeTrigger(self.triggerId);

    for sprayer,_ in pairs(self.triggeredSowingMachines) do
        if not sprayer.isDeleted then
            sprayer:removeSprayerFillTrigger(self);
        end
    end
end;

function FertilizerFillTrigger:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    if onEnter or onLeave then
        local sprayer = g_currentMission.objectToTrailer[otherShapeId];

        if sprayer ~= nil and sprayer.addSprayerFillTrigger ~= nil and sprayer.removeSprayerFillTrigger ~= nil then
            if onEnter then
                if self.triggeredSowingMachines[sprayer] == nil then
					self.triggeredSowingMachines[sprayer] = sprayer;
					sprayer:addSprayerFillTrigger(self);
                end;
            else -- onLeave
				if self.triggeredSowingMachines[sprayer] ~= nil then
					sprayer:removeSprayerFillTrigger(self);
					self.triggeredSowingMachines[sprayer] = nil;
				end;
            end;
        end;
    end;
end;

function FertilizerFillTrigger:fillSprayer(sprayer, delta)

    local fillType = self.fillType;
    local priceScale = 0.9;
    if not sprayer:allowFillType(self.fillType, false) then
        return 0.0;
    end
    local oldFillLevel = sprayer:getFillLevel(fillType);
    sprayer:setFillLevel(oldFillLevel + delta, fillType, true);
    delta = sprayer:getFillLevel(fillType) - oldFillLevel;

    local fillTypeDesc = Fillable.fillTypeIndexToDesc[self.fillType]
    if fillTypeDesc ~= nil then

        local price = delta*fillTypeDesc.pricePerLiter*priceScale;
        g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + price;
        g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + price;

        g_currentMission:addSharedMoney(-price, "other");
    end

    return delta;
end

function FertilizerFillTrigger:getIsActivatable(fillable)
    if not fillable:allowFillType(self.fillType, false) then
        return false;
    end;
    return true;
end;